import numpy as np
import torch
from logger import Logger
LOGGER = Logger().logger()

class Loss:
    @staticmethod
    def content_loss(F,P):
        '''
        Function to compute content loss between two feature representations at a particular layer
        @params
        F: 2D numpy array
        P: 2D numpy array
        Author: @meetdoshi
        '''
        l2_norm_sq = None
        for i in range(len(F)):
            try:
                diff = F[i]-P[i]
                l2_norm_sq = torch.mean(diff**2)
            except Exception as e:
                LOGGER.error("Error computing loss",e)
        return l2_norm_sq

    @staticmethod
    def gram_matrix(F):
        '''
        Function to compute the gram matrix of a feature representation at a layer
        Author: @himalisaini
        '''
        shape_mat = F.shape
        num_channels = shape_mat[1]
        height = shape_mat[2]
        width = shape_mat[3]
        return torch.mm(F.view(num_channels,(height*width)),F.view(num_channels,(height*width)).t())

    @staticmethod
    def style_loss(F,A):
        '''
        Function to compute style loss between two feature representations at multiple layers
        @params
        Author: @soumyagupta
        '''
        for i in range(len(F)):
            num_channels = F[i][1]
            h = F[i][2]
            w = F[i][3]
            style_gram_matrix = Loss.gram_matrix(F[i])
            target_gram_matrix = Loss.gram_matrix(A[i])
            loss_s = torch.sum((style_gram_matrix-target_gram_matrix)**2)
            constant = 1/(4.0*(num_channels**2)*((h*w)**2))
        return constant*loss_s

    @staticmethod
    def total_loss(alpha,beta,cont_fmap_real,cont_fmap_noise,style_fmap_real,style_fmap_noise):
        '''
        Function which computes total loss and returns it
        @params
        Author: @jiteshg
        '''
        content_loss = Loss.content_loss(cont_fmap_real, cont_fmap_noise)
        style_loss = Loss.style_loss(style_fmap_real, style_fmap_noise)
        loss_t = alpha*content_loss + beta*style_loss
        return loss_t